"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const colors = require("colors/safe");
const moment = require("moment");
const _types_1 = require("../_types");
const aws_client_1 = require("../aws-client");
const find_cluster_1 = require("../find-cluster");
const taskRegex = /task\/(.+)$/;
class Logs extends command_1.Command {
    async run() {
        // tslint:disable-next-line:no-shadowed-variable
        const { args, flags } = this.parse(Logs);
        console.log(`${colors.green('... connecting AWS CloudWatchLogs ...')}`);
        const ecs = aws_client_1.getECSInstance(flags.region);
        const { clusterArn, clusterName, taskArns } = await find_cluster_1.default(ecs, {
            family: flags.taskDefinition,
        });
        const taskIds = taskArns
            .map((taskArn) => {
            const matches = taskArn.match(taskRegex);
            if (matches && matches.length >= 2) {
                return matches[1];
            }
        })
            .filter((taskId) => taskId && taskId.length > 0);
        const logStreamNames = taskIds.map((taskId) => {
            if (flags.containerName) {
                return `${clusterName}/${flags.containerName}/${taskId}`;
            }
            return `${clusterName}/${flags.taskDefinition}/${taskId}`;
        });
        const cloudwatchlogs = aws_client_1.getCloudWatchLogsInstance(flags.region);
        let lastTime = moment()
            .add(-5, 'minutes')
            .valueOf();
        const tailLogs = async (startTime, endTime) => {
            const params = {
                filterPattern: args.filter,
                interleaved: true,
                logGroupName: flags.groupName,
                logStreamNames,
                startTime,
            };
            if (endTime) {
                params.endTime = endTime;
            }
            const tailData = await cloudwatchlogs.filterLogEvents(params).promise();
            if (tailData && tailData.events && tailData.events.length > 0) {
                tailData.events.forEach((x) => {
                    console.log(`${colors.yellow(moment(x.timestamp).toISOString())}  ${x.message}`);
                });
                return tailData.events[tailData.events.length - 1].timestamp + 1;
            }
            return startTime;
        };
        if (flags.follow) {
            setInterval(async () => {
                lastTime = await tailLogs(lastTime);
            }, 1000);
        }
        else {
            await tailLogs(moment(flags.startTime).valueOf(), moment(flags.endTime).valueOf());
        }
    }
}
Logs.description = 'view ecs logs';
Logs.examples = [
    `## follow ops logs
$ ee logs -f -t ops`,
    `## ops logs from 2018-12-31T09:00:00Z to 2018-12-31T09:10:00
$ ee logs -t ops -s 2018-12-31T09:00:00 -e 2018-12-31T09:10:00`,
    `## follow ops logs, filtered by "health"
$ ee logs -f -t ops 'health'`,
];
Logs.flags = Object.assign({}, _types_1.defaultFlags, { containerName: command_1.flags.string({ char: 'o' }), endTime: command_1.flags.string({ char: 'e', default: moment().toISOString() }), follow: command_1.flags.boolean({ char: 'f' }), groupName: command_1.flags.string({ char: 'g', default: '/aws/ecs', required: true }), startTime: command_1.flags.string({
        char: 's',
        default: moment()
            .add(-5, 'minutes')
            .toISOString(),
    }) });
Logs.args = [{ name: 'filter', required: false }];
exports.default = Logs;
//# sourceMappingURL=logs.js.map