"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = require("aws-sdk");
const moment = require("moment");
const util = require("util");
const find_cluster_1 = require("./find-cluster");
async function runTask(ecs, { taskDefinition: familyPrefix, cluster, memoryReservation, command, }) {
    const { taskDefinitionArns: taskDefinitions } = await ecs
        .listTaskDefinitions({ familyPrefix, sort: 'DESC' })
        .promise();
    if (!taskDefinitions || taskDefinitions.length === 0) {
        throw new Error('No task definitions are found');
    }
    const { taskDefinition: latestTaskDefinitionDescription, } = await ecs
        .describeTaskDefinition({ taskDefinition: taskDefinitions[0] })
        .promise();
    const { name, logConfiguration, } = latestTaskDefinitionDescription.containerDefinitions[0];
    const runningCluster = cluster ||
        (await find_cluster_1.default(ecs, latestTaskDefinitionDescription)).clusterArn;
    const containerOverride = [
        memoryReservation
            ? { name, command, memoryReservation }
            : { name, command },
    ];
    const { failures, tasks } = await ecs
        .runTask({
        cluster: runningCluster,
        overrides: {
            containerOverrides: containerOverride,
        },
        taskDefinition: taskDefinitions[0],
    })
        .promise();
    if (failures && failures.length > 0) {
        for (const failure of failures) {
            console.error(failure);
        }
        throw new Error('Failed to initiate task');
    }
    const { taskArn } = tasks[0];
    const uuid = taskArn.match(/[^/]+$/);
    if (uuid &&
        uuid[0] &&
        logConfiguration &&
        logConfiguration.logDriver === 'awslogs') {
        await waitAndLogTask(ecs, {
            cluster: runningCluster,
            logGroupName: logConfiguration.options['awslogs-group'],
            logStreamName: `${logConfiguration.options['awslogs-stream-prefix']}/${name}/${uuid}`,
            taskArn: taskArn,
        });
    }
}
exports.default = runTask;
async function waitAndLogTask(ecs, { cluster, taskArn, logGroupName, logStreamName, }) {
    while (true) {
        const { tasks } = await ecs
            .describeTasks({ cluster, tasks: [taskArn] })
            .promise();
        if (tasks.length > 0) {
            break;
        }
        await sleep(1);
    }
    const cloudWatchLogs = new aws_sdk_1.CloudWatchLogs();
    let token = null;
    let lastStatus = null;
    while (lastStatus !== 'STOPPED') {
        const { tasks: newTasks } = await ecs
            .describeTasks({ cluster, tasks: [taskArn] })
            .promise();
        lastStatus = newTasks.length > 0 ? newTasks[0].lastStatus : 'STOPPED';
        await sleep(1);
        try {
            const data = await (token
                ? cloudWatchLogs
                    .getLogEvents({
                    logGroupName,
                    logStreamName,
                    nextToken: token,
                    startFromHead: true,
                })
                    .promise()
                : cloudWatchLogs
                    .getLogEvents({ logGroupName, logStreamName, startFromHead: true })
                    .promise());
            data.events.forEach(({ timestamp, message }) => console.log(util.format('[%s] %s', moment(timestamp).toISOString(), message)));
            token = data.nextForwardToken;
        }
        catch (_a) {
            // noop
        }
    }
}
function sleep(seconds) {
    return new Promise((resolve) => {
        setTimeout(resolve, seconds * 1000);
    });
}
//# sourceMappingURL=run-task.js.map